from .random import RandomSampling
from .uncertainty import LeastConfidence, LeastConfidence, MarginConfidence, RatioConfidence, EntropySampling, \
    BayesianDisagreement, MeanSTDSampling, VarRatioSampling
from .cluster import KMeansSampling
from .diversity import BadgeSampling
